# flake8: noqa
# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: blockchain_txn.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


import hm_pyhelper.protos.blockchain_txn_add_gateway_v1_pb2 as blockchain__txn__add__gateway__v1__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x14\x62lockchain_txn.proto\x12\x06helium\x1a#blockchain_txn_add_gateway_v1.proto\"U\n\x0e\x62lockchain_txn\x12<\n\x0b\x61\x64\x64_gateway\x18\x01 \x01(\x0b\x32%.helium.blockchain_txn_add_gateway_v1H\x00\x42\x05\n\x03txnb\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'blockchain_txn_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _BLOCKCHAIN_TXN._serialized_start=69
  _BLOCKCHAIN_TXN._serialized_end=154
# @@protoc_insertion_point(module_scope)
